@@ -106,9 +106,11 @@ rabbit_opts = [
                 ),
     cfg.FloatOpt('kombu_reconnect_delay',
                  default=1.0,
+                 min=0.0,
+                 max=amqpdriver.ACK_REQUEUE_EVERY_SECONDS_MAX * 0.9,
                  deprecated_group='DEFAULT',
-                 help='How long to wait before reconnecting in response to an '
-                      'AMQP consumer cancel notification.'),
+                 help='How long to wait (in seconds) before reconnecting in '
+                      'response to an AMQP consumer cancel notification.'),
     cfg.StrOpt('kombu_compression',
                help="EXPERIMENTAL: Possible values are: gzip, bz2. If not "
                     "set compression will not be used. This option may not "
@@ -174,7 +176,8 @@ rabbit_opts = [
                'or dead-lettered (if a DLX exchange has been configured) '
                'Used only when rabbit_quorum_queue is enabled, '
                'Default 0 which means dont set a limit.'),
-    cfg.IntOpt('rabbit_quroum_max_memory_length',
+    cfg.IntOpt('rabbit_quorum_max_memory_length',
+               deprecated_name='rabbit_quroum_max_memory_length',
                default=0,
                help='By default all messages are maintained in memory '
                'if a quorum queue grows in length it can put memory '
@@ -182,7 +185,8 @@ rabbit_opts = [
                'of messages in the quorum queue. '
                'Used only when rabbit_quorum_queue is enabled, '
                'Default 0 which means dont set a limit.'),
-    cfg.IntOpt('rabbit_quroum_max_memory_bytes',
+    cfg.IntOpt('rabbit_quorum_max_memory_bytes',
+               deprecated_name='rabbit_quroum_max_memory_bytes',
                default=0,
                help='By default all messages are maintained in memory '
                'if a quorum queue grows in length it can put memory '
@@ -349,11 +353,44 @@ class Consumer(object):
         self._declared_on = None
         self.exchange = kombu.entity.Exchange(
             name=exchange_name,
-            type=type,
+            type=self.type,
             durable=self.durable,
             auto_delete=self.exchange_auto_delete)
         self.enable_cancel_on_failover = enable_cancel_on_failover
 
+    def _declare_fallback(self, err, conn, consumer_arguments):
+        """Fallback by declaring a non durable queue.
+
+        When a control exchange is shared between services it is possible
+        that some service created first a non durable control exchange and
+        then after that an other service can try to create the same control
+        exchange but as a durable control exchange. In this case RabbitMQ
+        will raise an exception (PreconditionFailed), and then it will stop
+        our execution and our service will fail entirly. In this case we want
+        to fallback by creating a non durable queue to match the default
+        config.
+        """
+        if "PRECONDITION_FAILED - inequivalent arg 'durable'" in str(err):
+            LOG.info(
+                "[%s] Retrying to declare the exchange (%s) as "
+                "non durable", conn.connection_id, self.exchange_name)
+            self.exchange = kombu.entity.Exchange(
+                name=self.exchange_name,
+                type=self.type,
+                durable=False,
+                auto_delete=self.queue_auto_delete)
+            self.queue = kombu.entity.Queue(
+                name=self.queue_name,
+                channel=conn.channel,
+                exchange=self.exchange,
+                durable=False,
+                auto_delete=self.queue_auto_delete,
+                routing_key=self.routing_key,
+                queue_arguments=self.queue_arguments,
+                consumer_arguments=consumer_arguments
+            )
+            self.queue.declare()
+
     def declare(self, conn):
         """Re-declare the queue after a rabbit (re)connect."""
 
@@ -376,7 +413,18 @@ class Consumer(object):
         try:
             LOG.debug('[%s] Queue.declare: %s',
                       conn.connection_id, self.queue_name)
-            self.queue.declare()
+            try:
+                self.queue.declare()
+            except amqp_exec.PreconditionFailed as err:
+                # NOTE(hberaud): This kind of exception may be triggered
+                # when a control exchange is shared between services and
+                # when services try to create it with configs that differ
+                # from each others. RabbitMQ will reject the services
+                # that try to create it with a configuration that differ
+                # from the one used first.
+                LOG.warning(err)
+                self._declare_fallback(err, conn, consumer_arguments)
+
         except conn.connection.channel_errors as exc:
             # NOTE(jrosenboom): This exception may be triggered by a race
             # condition. Simply retrying will solve the error most of the time
@@ -605,6 +653,8 @@ class Connection(object):
             # if it was already monkey patched by eventlet/greenlet.
             global threading
             threading = _utils.stdlib_threading
+            amqpdriver.threading = _utils.stdlib_threading
+            amqpdriver.queue = _utils.stdlib_queue
 
         self.direct_mandatory_flag = driver_conf.direct_mandatory_flag
 
@@ -791,8 +841,8 @@ class Connection(object):
     def _get_quorum_configurations(self, driver_conf):
         """Get the quorum queue configurations"""
         delivery_limit = driver_conf.rabbit_quorum_delivery_limit
-        max_memory_length = driver_conf.rabbit_quroum_max_memory_length
-        max_memory_bytes = driver_conf.rabbit_quroum_max_memory_bytes
+        max_memory_length = driver_conf.rabbit_quorum_max_memory_length
+        max_memory_bytes = driver_conf.rabbit_quorum_max_memory_bytes
         return QuorumMemConfig(delivery_limit, max_memory_length,
                                max_memory_bytes)
 
@@ -1354,7 +1404,20 @@ class Connection(object):
         """Publish a message."""
 
         if not (exchange.passive or exchange.name in self._declared_exchanges):
-            exchange(self.channel).declare()
+            try:
+                exchange(self.channel).declare()
+            except amqp_exec.PreconditionFailed as err:
+                # NOTE(hberaud): This kind of exception may be triggered
+                # when a control exchange is shared between services and
+                # when services try to create it with configs that differ
+                # from each others. RabbitMQ will reject the services
+                # that try to create it with a configuration that differ
+                # from the one used first.
+                if "PRECONDITION_FAILED - inequivalent arg 'durable'" \
+                   in str(err):
+                    LOG.warning("Force creating a non durable exchange.")
+                    exchange.durable = False
+                    exchange(self.channel).declare()
             self._declared_exchanges.add(exchange.name)
 
         log_info = {'msg': msg,
